/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.utils;

import org.fortiss.tooling.kernel.model.constraints.ConstraintsFactory;
import org.fortiss.tooling.kernel.model.constraints.ErrorVerificationStatus;
import org.fortiss.tooling.kernel.model.constraints.FailVerificationStatus;
import org.fortiss.tooling.kernel.model.constraints.IConstrained;
import org.fortiss.tooling.kernel.model.constraints.IConstraint;
import org.fortiss.tooling.kernel.model.constraints.OutdatedVerificationStatus;
import org.fortiss.tooling.kernel.model.constraints.SuccessVerificationStatus;

/**
 * Utility methods for dealing with constraints.
 * 
 * @author aravantinos
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 1325BAE0A0C28B3F093D89D912BE1AFF
 */
public class ConstraintsUtils {
	/**
	 * @param c
	 * @param clazz
	 * @return <code>true</code> if <code>c</code> contains a constraint of type <code>clazz</code>.
	 */
	public static boolean containsConstraintOfType(IConstrained c,
			Class<? extends IConstraint> clazz) {
		return getConstraintOfType(c, clazz) != null;
	}

	/**
	 * @param c
	 * @param clazz
	 * @return the first constraint of <code>c</code> of type <code>clazz</code>.
	 */
	public static <T extends IConstraint> T getConstraintOfType(IConstrained c, Class<T> clazz) {
		return EcoreUtils.pickFirstInstanceOf(clazz, c.getConstraints());
	}

	/** Exception which embeds an error. */
	public static class ErrorEmbeddingException extends RuntimeException {

		/** The embedded error. */
		public ErrorVerificationStatus error;

		/** Constructor. */
		public ErrorEmbeddingException(ErrorVerificationStatus error) {
			super();
			this.error = error;
		}
	}

	/**
	 * @param c
	 * @return A "success" verification status for <code>c</code>.
	 */
	public static SuccessVerificationStatus createSuccessVerificationStatus(IConstraint c) {
		SuccessVerificationStatus s =
				ConstraintsFactory.eINSTANCE.createSuccessVerificationStatus();
		c.setVerificationStatus(s);
		return s;
	}

	/**
	 * @param c
	 * @return A "outdated" verification status for <code>c</code>.
	 */
	public static OutdatedVerificationStatus createOutdatedVerificationStatus(IConstraint c) {
		OutdatedVerificationStatus status =
				ConstraintsFactory.eINSTANCE.createOutdatedVerificationStatus();
		c.setVerificationStatus(status);
		return status;
	}

	/**
	 * @param c
	 * @return A "fail" verification status for <code>c</code>.
	 */
	public static FailVerificationStatus createFailVerificationStatus(IConstraint c) {
		FailVerificationStatus status = ConstraintsFactory.eINSTANCE.createFailVerificationStatus();
		c.setVerificationStatus(status);
		return status;
	}

	/**
	 * @param c
	 * @return An "error" verification status for <code>c</code>.
	 */
	public static ErrorVerificationStatus createErrorVerificationStatus(IConstraint c) {
		ErrorVerificationStatus status =
				ConstraintsFactory.eINSTANCE.createErrorVerificationStatus();
		c.setVerificationStatus(status);
		return status;
	}
}
